@@ -22,10 +22,10 @@ PIG_DEFINE_STUB_TYPE(pig_type_qt_ubits, uchar *)
 PIG_DEFINE_STUB_TYPE(pig_type_qt_ubitsarray, uchar **)
 PIG_DEFINE_STUB_TYPE(pig_type_qt_uintarray, uint **)
 PIG_DEFINE_STUB_TYPE(pig_type_qt_VHorientation, int)
-PIG_DEFINE_STUB_TYPE(pig_type_qt_QString_ref, class QString &)
 PIG_DEFINE_STUB_TYPE(pig_type_qt_QArray_QRect_ptr, void *)
 PIG_DEFINE_STUB_TYPE(pig_type_qt_QFileInfo_ptr, class QFileInfo *)
 PIG_DEFINE_STUB_TYPE(pig_type_qt_QFileInfoList_ptr, class QFileInfoList *)
+PIG_DEFINE_STUB_TYPE(pig_type_qt_QStringList_ptr, class QStringList *)
 PIG_DEFINE_STUB_TYPE(pig_type_qt_QStrList_ptr, class QStrList *)
 PIG_DEFINE_STUB_TYPE(pig_type_qt_QTabList_ptr, class QTabList *)
 
@@ -226,6 +226,56 @@ PIG_DEFINE_STUB_PUSH(pig_type_qt_xpm, const char **)
 PIG_DEFINE_STUB_POP(pig_type_qt_xpm, const char **)
 
 
+PIG_DEFINE_SCOPE_ARGUMENT(pig_type_qt_const_QString_ref) {
+    delete (QString *)pig0;
+}
+
+PIG_DEFINE_SCOPE_VIRTUAL(pig_type_qt_const_QString_ref) {
+    delete (QString *)pig0;
+}
+
+PIG_DEFINE_TYPE_ARGUMENT(pig_type_qt_const_QString_ref, const QString &) {
+    PIGARGS;
+    QString *pigr = new QString;
+    if(SvOK(PIG_ARG)) {
+	STRLEN n_a;
+	*pigr = QString(SvPV(PIG_ARG, n_a));
+    }
+    PIGSCOPE_ARGUMENT(pig_type_qt_const_QString_ref, pigr);
+    PIGARGUMENT(*pigr);
+}
+
+PIG_DEFINE_TYPE_DEFARGUMENT(pig_type_qt_const_QString_ref, const QString &) {
+    PIGARGS;
+    QString *pigr = new QString;
+    if(PIG_ARGOK) {
+        STRLEN n_a;
+        *pigr = QString(SvPV(PIG_ARG, n_a));
+    }
+    PIGSCOPE_ARGUMENT(pig_type_qt_const_QString_ref, pigr);
+    PIGARGUMENT(*pigr);
+}
+
+PIG_DEFINE_TYPE_RETURN(pig_type_qt_const_QString_ref, const QString &) {
+    PIGRET;
+    PIGRETURN(sv_2mortal(!pig0.isNull() ?
+                         newSVpv((char *)pig0.data(), 0) :
+                         newSVsv(&PIGsv_undef)));
+}
+
+PIG_DEFINE_STUB_PUSH(pig_type_qt_const_QString_ref, const QString &)
+
+PIG_DEFINE_TYPE_POP(pig_type_qt_const_QString_ref, const QString &) {
+    PIGPOPSTACK;
+    QString *pigr;
+    if(SvOK(PIG_TOPSTACK)) {
+        STRLEN n_a;
+        *pigr = QString(SvPV(PIG_TOPSTACK, n_a));
+    }
+    PIGSCOPE_VIRTUAL(pig_type_qt_const_QString_ref, pigr);
+    PIGPOP(*pigr);
+}
+
 
 PIG_DEFINE_SCOPE_ARGUMENT(pig_type_qt_QString_ptr) {
     delete (QString *)pig0;
@@ -261,7 +311,7 @@ PIG_DEFINE_TYPE_DEFARGUMENT(pig_type_qt_QString_ptr, QString *) {
 PIG_DEFINE_TYPE_RETURN(pig_type_qt_QString_ptr, QString *) {
     PIGRET;
     PIGRETURN(sv_2mortal(pig0 && !pig0->isNull() ?
-			 newSVpv(pig0->data(), 0) :
+			 newSVpv((char *)pig0->data(), 0) :
 			 newSVsv(&PIGsv_undef)));
 }
 
@@ -397,7 +447,7 @@ PIG_DEFINE_TYPE(pig_type_qt_pointarrayitems)
 PIG_DEFINE_TYPE(pig_type_qt_HVorientation)
 PIG_DEFINE_TYPE(pig_type_qt_VHorientation)
 PIG_DEFINE_TYPE(pig_type_qt_QString_ptr)
-PIG_DEFINE_TYPE(pig_type_qt_QString_ref)
+PIG_DEFINE_TYPE(pig_type_qt_const_QString_ref)
 PIG_DEFINE_TYPE(pig_type_qt_QByteArray_ptr)
 PIG_DEFINE_TYPE(pig_type_qt_QArray_QRect_ptr)
 PIG_DEFINE_TYPE(pig_type_qt_QFileInfo_ptr)
@@ -424,7 +474,7 @@ PIG_EXPORT_TABLE(pigtype_qt)
     PIG_EXPORT_TYPE(pig_type_qt_HVorientation, "Qt HVorientation")
     PIG_EXPORT_TYPE(pig_type_qt_VHorientation, "Qt VHorientation")
     PIG_EXPORT_TYPE(pig_type_qt_QString_ptr, "Qt QString*")
-    PIG_EXPORT_TYPE(pig_type_qt_QString_ref, "Qt QString&")
+    PIG_EXPORT_TYPE(pig_type_qt_const_QString_ref, "Qt const QString&")
     PIG_EXPORT_TYPE(pig_type_qt_QByteArray_ptr, "Qt QByteArray*")
     PIG_EXPORT_TYPE(pig_type_qt_QArray_QRect_ptr, "Qt QArray<QRect>*")
     PIG_EXPORT_TYPE(pig_type_qt_QFileInfo_ptr, "Qt QFileInfo*")
